#====================================================
#=== Validation of stepwise-based procedure in GAMLSS
#=== by Thiago G. Ramires et al.
#====================================================

# ZIP distribution (linear structure)

#note: change penalty on line 25
library(gamlss)
library(gamlss.dist)

#saving selected parms
mus150<-sigs150<-c()  
mus300<-sigs300<-c()  
pmu150<-psig150<-c()
pmu300<-psig300<-c()
nn<-c(150,300)


for(i in 1:1000){
  for(k in 1:2){ #sample size

    n=nn[k]
    #change penalty kkk=2 or kkk=log(n) or kkk=((log(n)+2)/2)
    kkk=2
    
    x1<-rbinom(n,size = 1,prob = .5) 
    x2<-rbinom(n,size = 1,prob = .5) 
    x3<-rbinom(n,size = 1,prob = .5) 
    x4<-rbinom(n,size = 1,prob = .5) 
    
    x5<-runif(n) 
    x6<-runif(n) 
    x7<-runif(n) 
    x8<-runif(n) 
    
    #model structure
    mu<- exp(.4+0.4*x1+.04*x3+.04*x5+.05*x7)
    xb=-2.11+.75*x2+1.85*x3+0.5*x6+.63*x7
    sig<-exp(xb)/(1+exp(xb)) #logistic
      
    #generating random samples
    y<-c()
    for(j in 1:n){
      y<-c(y,rZIP(1,mu[j],sig[j]))
    }
    
    
    #model selection
    m0<-gamlss(y~1,n.cyc=300,c.crit=.0001,family = ZIP)
    m2<-stepGAICAll.A(m0,k=kkk, scope=list(lower=~1,upper=~x1+x2+x3+x4+x5+x6+x7+x8),parallel = 'snow')
    
    a=summary(m2)

    #saving results sample size = 150
    if(k==1) pmu150<-c(pmu150,a[1:length(m2$mu.coefficients),4])
    if(k==1) psig150<-c(psig150,a[(length(m2$mu.coefficients)+1):length(a[,1]),4])
    if(k==1) mus150<-c(mus150,m2$mu.coefficients)
    if(k==1) sigs150<-c(sigs150,m2$sigma.coefficients)
    #saving results sample size = 300
    if(k==2) pmu300<-c(pmu300,a[1:length(m2$mu.coefficients),4])
    if(k==2) psig300<-c(psig300,a[(length(m2$mu.coefficients)+1):length(a[,1]),4])
    if(k==2) mus300<-c(mus300,m2$mu.coefficients)
    if(k==2) sigs300<-c(sigs300,m2$sigma.coefficients)
    
  } # for k end here
} # for i end here 
                                                                                         ###### barplot #####

(um<-(table(labels(mus150))*(100/1000))[-1])
(dois<-(table(labels(sigs150))*(100/1000))[-1])
(tres<-(table(labels(mus300))*(100/1000))[-1])
(quatro<-(table(labels(sigs300))*(100/1000))[-1])

acurace<-c(um,tres,dois,quatro) # mu150 mu300, sig150 sig300 length=32
variaveis<-c(rep(c('x1','x2','x3','x4','x5','x6','x7','x8'),4))
parametros<-c(rep('mu for n=150',8),rep('mu for n=300',8),
              rep('sigma for n=150',8),rep('sigma for n=300',8))

library(ggplot2)

colors <- c(
  rep(c("springgreen1","salmon"),4),                      #mu150
  rep(c("springgreen1","salmon"),4),                      #mu300                                 
  "salmon","springgreen1","springgreen1","salmon","salmon","springgreen1","springgreen1","salmon", #sig150
  "salmon","springgreen1","springgreen1","salmon","salmon","springgreen1","springgreen1","salmon" #sig300
)           

colors2<-ifelse(colors=="springgreen1","correct",'wrong')
data_long<-data.frame(acurace,variaveis,parametros)

a=ggplot(data_long, aes(x = variaveis, y = acurace, fill = colors2, width=0.85)) + 
  labs(y = "Selected %", x = "Variables", fill = NULL) +
  geom_bar(stat = "identity") +
  facet_wrap(~parametros) +
  coord_flip() +
  scale_fill_manual(values = c("springgreen1","salmon"))+
  theme_bw() + theme(strip.background  = element_blank(),
                     panel.grid.major = element_line(colour = "grey80"),
                     panel.border = element_blank(),
                     axis.ticks = element_line(size = 0),
                     panel.grid.minor.y = element_blank(),
                     panel.grid.major.y = element_blank() ) +
  theme(legend.position="bottom") + scale_y_continuous(limits=c(0,100))

a


#### p-values ####
pmuall<-c(pmu150,pmu300)
psigall<-c(psig150,psig300)
nmu=c(rep(150,length(pmu150)),rep(300,length(pmu300)))
nsigma=c(rep(150,length(psig150)),rep(300,length(psig300)))

dadosmu<-data.frame(variables=labels(pmuall),pvalue=pmuall,n=nmu)
dadossig<-data.frame(variables=labels(psigall),pvalue=psigall,n=nsigma)

dadosmu=subset(dadosmu,variables!="(Intercept)" & variables!="")
dadossig=subset(dadossig,variables!="(Intercept)" & variables!="")

a=ggplot(dadosmu, aes(x=variables,y=pvalue,fill=variables)) + geom_boxplot() + 
  facet_wrap(~n)+theme_minimal()+theme(legend.position="none")+labs(title=expression(mu))+ylim(0,0.15)

b=ggplot(dadossig, aes(x=variables,y=pvalue,fill=variables)) + geom_boxplot() + 
  facet_wrap(~n)+theme_minimal()+theme(legend.position="none")+labs(title=expression(sigma))+ylim(0,0.15)

library(gridExtra)
grid.arrange(a, b, ncol=2)




########################
# changing slope value#
#######################
library(gamlss)

parms<-size<-musig<-slope<-c()  
estat<-c()

nn<-c(150,300)
x5new=c(0.05,0.15,0.25,0.35,0.45) #changind slope mu
x6new=c(0.3,0.8,1.3,1.8,2.3)      #changind slope sigma

for(i in 1:100){        #simu
  for(k in 1:2){        #sample size
    for(o in 1:3){      #penalty
      for(b in 1:2){    #mu or sigma
        for(t in 1:5){  # slope value
          
          n=nn[k]
          kkkk=c(2,log(n),((log(n)+2)/2))
          kkk=kkkk[o]
          
          x1<-rbinom(n,size = 1,prob = .5) 
          x2<-rbinom(n,size = 1,prob = .5) 
          x3<-rbinom(n,size = 1,prob = .5) 
          x4<-rbinom(n,size = 1,prob = .5) 
          
          x5<-runif(n) 
          x6<-runif(n) 
          x7<-runif(n) 
          x8<-runif(n) 
          
          
          #simulation for mu
          if(b==1){
            mu<- exp(.4+0.4*x1+.04*x3+x5new[t]*x5+.05*x7)
            xb=-2.11+.75*x2+1.85*x3+0.5*x6+.63*x7
            sig<-exp(xb)/(1+exp(xb)) #logistic
          }      
          
          #simulation for sigma
          if(b==2){
            #model structure
            mu<- exp(.4+0.4*x1+.04*x3+.04*x5+.05*x7)
            xb=-2.11+.75*x2+1.85*x3+x6new[t]*x6+.63*x7
            sig<-exp(xb)/(1+exp(xb)) #logistic
          }      
          
          
          #generating random samples
          y<-c()
          for(j in 1:n){
            y<-c(y,rZIP(1,mu[j],sig[j]))
          }
          
          #model selection
          m0<-gamlss(y~1,n.cyc=300,c.crit=.0001,family = ZIP)
          m2<-stepGAICAll.A(m0,k=kkk, scope=list(lower=~1,upper=~x1+x2+x3+x4+x5+x6+x7+x8),parallel = 'snow')#ncpus = 2
          
          
          #1=selected, 0=not selected
          if(b==1) parms<-c(parms,ifelse(!is.na(match('x5',labels(m2$mu.coefficients))),1,0))
          if(b==2) parms<-c(parms,ifelse(!is.na(match('x6',labels(m2$sigma.coefficients))),1,0))
          
          if(o==1) estat=c(estat,"AIC")
          if(o==2) estat=c(estat,"BIC")
          if(o==3) estat=c(estat,"GAIC")
          
          if(k==1) size=c(size,150)
          if(k==2) size=c(size,300)
          
          if(b==1) musig=c(musig,'mu')
          if(b==2) musig=c(musig,'sig')
          
          if(b==1) slope=c(slope,x5new[t])
          if(b==2) slope=c(slope,x6new[t])
          
        }#t ends here (slope)
      }#b ends here (musig)
    } # for o ends here (penalty)
  } # for k end here
} # for i end here 


dados=data.frame(parms,estat,size,musig,slope)

dadosmu=subset(dados,musig=='mu')
dadossig=subset(dados,musig=='sig')


library(dplyr)
dadosmu2=dadosmu %>%
  group_by(estat,size,slope) %>%
  summarise_at(vars(parms), funs(sum))

dadossig2=dadossig %>%
  group_by(estat,size,slope) %>%
  summarise_at(vars(parms), funs(sum))

dadosmu2$correct=dadosmu2$parms/100
dadossig2$correct=dadossig2$parms/100

names(dadosmu2)[1]='Criterion'
dadosmu2$sample=as.factor(dadosmu2$size)

names(dadossig2)[1]='Criterion'
dadossig2$sample=as.factor(dadossig2$size)


library(plyr)
dadosmu2$Criterion=revalue(dadosmu2$Criterion, c('GAIC'='GAICav'))
dadossig2$Criterion=revalue(dadossig2$Criterion, c('GAIC'='GAICav'))

library(ggplot2)
a=ggplot(dadosmu2,aes(slope,correct))+
  geom_line(aes(linetype=sample,color=Criterion),lwd=1.4)+
  theme_minimal()+labs(title=expression(mu(beta[51])))+theme(legend.position="top")+
  ylim(0,1)

b=ggplot(dadossig2,aes(slope,correct))+
  geom_line(aes(linetype=sample,color=Criterion),lwd=1.4)+
  theme_minimal()+labs(title=expression(sigma(beta[62])))+theme(legend.position="non")+
  ylim(0,1)

library(gridExtra)
grid.arrange(a, b, ncol=1)
